package betterwithmods.common.blocks.tile;

import betterwithmods.common.BWSounds;
import betterwithmods.common.blocks.mechanical.BlockMechMachines;
import betterwithmods.common.registry.bulk.manager.MillManager;
import betterwithmods.common.registry.bulk.recipes.MillRecipe;
import betterwithmods.util.InvUtils;
import net.minecraft.block.Block;
import net.minecraft.block.state.IBlockState;
import net.minecraft.init.SoundEvents;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.ITickable;
import net.minecraft.util.NonNullList;
import net.minecraft.util.SoundCategory;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;

import java.util.ArrayList;
import java.util.List;

public class TileEntityMill extends TileBasicInventory implements ITickable {

    public static final int GRIND_TIME = 200;

    public int grindCounter;

    private int grindType = 0;
    private boolean validateContents;
    private boolean containsIngredientsToGrind;

    public TileEntityMill() {
        this.grindCounter = 0;
        this.validateContents = true;
    }

    @Override
    public boolean shouldRefresh(World world, BlockPos pos, IBlockState oldState, IBlockState newState) {
        return oldState.func_177230_c() != newState.func_177230_c();
    }

    @Override
    public void func_73660_a() {
        if (this.func_145831_w().field_72995_K)
            return;

        Block block = this.func_145831_w().func_180495_p(this.field_174879_c).func_177230_c();

        if (block == null || !(block instanceof BlockMechMachines))
            return;

        BlockMechMachines mill = (BlockMechMachines) block;

        if (this.validateContents)
            validateContents();

        if (mill.isMechanicalOn(func_145831_w(), field_174879_c))
            if (func_145831_w().field_73012_v.nextInt(20) == 0)
                func_145831_w().func_184133_a(null, field_174879_c, BWSounds.STONEGRIND, SoundCategory.BLOCKS, 0.5F + func_145831_w().field_73012_v.nextFloat() * 0.1F, 0.5F + func_145831_w().field_73012_v.nextFloat() * 0.1F);

        if (this.containsIngredientsToGrind && mill.isMechanicalOn(func_145831_w(), field_174879_c)) {
            if (!this.func_145831_w().field_72995_K) {
                if (grindType == 2) {
                    if (this.func_145831_w().field_73012_v.nextInt(25) < 2) {
                        func_145831_w().func_184133_a(null, field_174879_c, SoundEvents.field_187555_bJ, SoundCategory.BLOCKS, 1F, func_145831_w().field_73012_v.nextFloat() * 0.4F + 0.8F);
                    }
                } else if (grindType == 1) {
                    if (this.func_145831_w().field_73012_v.nextInt(20) < 2)
                        func_145831_w().func_184133_a(null, field_174879_c, SoundEvents.field_187863_gH, SoundCategory.BLOCKS, 2.0F, (func_145831_w().field_73012_v.nextFloat() - func_145831_w().field_73012_v.nextFloat()) * 0.2F + 1.0F);
                }
            }
            this.grindCounter++;
            if (this.grindCounter > GRIND_TIME - 1) {
                grindContents();
                this.grindCounter = 0;
                this.validateContents = true;
            }
        }
    }

    @Override
    public void func_145839_a(NBTTagCompound tag) {
        super.func_145839_a(tag);
        if (tag.func_74764_b("GrindCounter"))
            this.grindCounter = tag.func_74762_e("GrindCounter");
    }

    @Override
    public int getInventorySize() {
        return 3;
    }

    @Override
    public NBTTagCompound func_189515_b(NBTTagCompound tag) {
        super.func_189515_b(tag);
        tag.func_74768_a("GrindCounter", this.grindCounter);
        return tag;
    }

    public int getGrindType() {
        return this.grindType;
    }

    @Override
    public void func_70296_d() {
        super.func_70296_d();
        validateContents();
        if (this.func_145831_w() != null && !this.func_145831_w().field_72995_K) {
            if (grindType == 1)
                this.func_145831_w().func_184133_a(null, this.field_174879_c, SoundEvents.field_187871_gL, SoundCategory.BLOCKS, 1F, 1.0F);
            this.validateContents = true;
        }
    }

    private void ejectStack(ItemStack stack) {
        List<EnumFacing> validDirections = new ArrayList<>();
        for (EnumFacing facing : EnumFacing.field_176754_o) {
            IBlockState check = func_145831_w().func_180495_p(field_174879_c.func_177972_a(facing));
            if (check.func_177230_c().func_176200_f(func_145831_w(), field_174879_c.func_177972_a(facing)) || func_145831_w().func_175623_d(field_174879_c.func_177972_a(facing)))
                validDirections.add(facing);
        }

        if (validDirections.isEmpty()) {
            IBlockState down = func_145831_w().func_180495_p(field_174879_c.func_177972_a(EnumFacing.DOWN));
            if (down.func_177230_c().func_176200_f(func_145831_w(), field_174879_c.func_177972_a(EnumFacing.DOWN)) || func_145831_w().func_175623_d(field_174879_c.func_177972_a(EnumFacing.DOWN)))
                validDirections.add(EnumFacing.DOWN);
        }

        BlockPos offset;
        if (validDirections.size() > 1)
            offset = field_174879_c.func_177972_a(validDirections.get(func_145831_w().field_73012_v.nextInt(validDirections.size())));
        else if (validDirections.isEmpty())
            offset = field_174879_c.func_177972_a(EnumFacing.UP);
        else
            offset = field_174879_c.func_177972_a(validDirections.get(0));

        InvUtils.ejectStackWithOffset(func_145831_w(), offset, stack);
    }

    public double getGrindProgress() {
        return this.grindCounter / (double)GRIND_TIME;
    }

    public boolean isGrinding() {
        return this.grindCounter > 0;
    }

    private boolean grindContents() {
        MillManager mill = MillManager.getInstance();
        List<Object> ingredients = mill.getValidCraftingIngredients(inventory);

        if (ingredients != null) {
            if (grindType == 1)
                this.func_145831_w().func_184133_a(null, field_174879_c, SoundEvents.field_187859_gF, SoundCategory.BLOCKS, 1.0F, 1.0F);
            NonNullList<ItemStack> output = mill.craftItem(field_145850_b,this,inventory);
            if (!output.isEmpty()) {
                for (ItemStack anOutput : output) {
                    ItemStack stack = anOutput.func_77946_l();
                    if (!stack.func_190926_b())
                        ejectStack(stack);
                }
            }
            return true;
        }
        return false;
    }

    private void validateContents() {
        int oldGrindType = getGrindType();
        int newGrindType = 0;
        MillRecipe recipe = MillManager.getInstance().getMostValidRecipe(inventory);
        if (recipe != null) {
            this.containsIngredientsToGrind = true;
            newGrindType = recipe.getGrindType();
        } else {
            this.grindCounter = 0;
            this.containsIngredientsToGrind = false;
        }
        this.validateContents = false;
        if (oldGrindType != newGrindType) {
            this.grindType = newGrindType;
        }
    }

}
